"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.semantic_model_update_suggestion import (
    SemanticModelUpdateSuggestion,
    SemanticModelUpdateSuggestionModel,
)
from snowflake.core.cortex.analyst_service._generated.models.warning import Warning, WarningModel


class GenerateFiltersAndMetricsSuggestionsResponse(BaseModel):
    """A model object representing the GenerateFiltersAndMetricsSuggestionsResponse resource.

    Constructs an object of type GenerateFiltersAndMetricsSuggestionsResponse with the provided properties.

    Parameters
    __________
    request_id : str, optional
        Unique request ID
    suggestions : list[SemanticModelUpdateSuggestion], optional

    warnings : list[Warning], optional

    experimental : str, optional
        JSON serialized string of experimental API fields (undocumented).
    """

    request_id: Optional[StrictStr] = None

    suggestions: Optional[List[SemanticModelUpdateSuggestion]] = None

    warnings: Optional[List[Warning]] = None

    experimental: Optional[StrictStr] = None

    __properties = ["request_id", "suggestions", "warnings", "experimental"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GenerateFiltersAndMetricsSuggestionsResponse:
        """Create an instance of GenerateFiltersAndMetricsSuggestionsResponse from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in suggestions (list)
        _items = []
        if self.suggestions:
            for _item in self.suggestions:
                if _item:
                    _items.append(_item.to_dict())
            _dict["suggestions"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in warnings (list)
        _items = []
        if self.warnings:
            for _item in self.warnings:
                if _item:
                    _items.append(_item.to_dict())
            _dict["warnings"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> GenerateFiltersAndMetricsSuggestionsResponse:
        """Create an instance of GenerateFiltersAndMetricsSuggestionsResponse from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return GenerateFiltersAndMetricsSuggestionsResponse.model_validate(obj)

        _obj = GenerateFiltersAndMetricsSuggestionsResponse.model_validate(
            {
                "request_id": obj.get("request_id"),
                "suggestions": [SemanticModelUpdateSuggestion.from_dict(_item) for _item in obj.get("suggestions")]
                if obj.get("suggestions") is not None
                else None,
                "warnings": [Warning.from_dict(_item) for _item in obj.get("warnings")]
                if obj.get("warnings") is not None
                else None,
                "experimental": obj.get("experimental"),
            }
        )

        return _obj


class GenerateFiltersAndMetricsSuggestionsResponseModel:
    def __init__(
        self,  # optional properties
        request_id: Optional[str] = None,
        suggestions: Optional[list[SemanticModelUpdateSuggestion]] = None,
        warnings: Optional[list[Warning]] = None,
        experimental: Optional[str] = None,
    ):
        """A model object representing the GenerateFiltersAndMetricsSuggestionsResponse resource.

        Constructs an object of type GenerateFiltersAndMetricsSuggestionsResponse with the provided properties.

        Parameters
        __________
        request_id : str, optional
            Unique request ID
        suggestions : list[SemanticModelUpdateSuggestion], optional

        warnings : list[Warning], optional

        experimental : str, optional
            JSON serialized string of experimental API fields (undocumented).
        """
        self.request_id = request_id
        self.suggestions = suggestions
        self.warnings = warnings
        self.experimental = experimental

    __properties = ["request_id", "suggestions", "warnings", "experimental"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return GenerateFiltersAndMetricsSuggestionsResponse(
            request_id=self.request_id,
            suggestions=[x._to_model() for x in self.suggestions] if self.suggestions is not None else None,
            warnings=[x._to_model() for x in self.warnings] if self.warnings is not None else None,
            experimental=self.experimental,
        )

    @classmethod
    def _from_model(cls, model) -> GenerateFiltersAndMetricsSuggestionsResponseModel:
        return GenerateFiltersAndMetricsSuggestionsResponseModel(
            request_id=model.request_id,
            suggestions=[SemanticModelUpdateSuggestionModel._from_model(x) for x in model.suggestions]
            if model.suggestions
            else None,
            warnings=[WarningModel._from_model(x) for x in model.warnings] if model.warnings else None,
            experimental=model.experimental,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a GenerateFiltersAndMetricsSuggestionsResponse.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the GenerateFiltersAndMetricsSuggestionsResponse object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> GenerateFiltersAndMetricsSuggestionsResponseModel:
        """Create an instance of GenerateFiltersAndMetricsSuggestionsResponse from a dict.

        This method constructs a GenerateFiltersAndMetricsSuggestionsResponse object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        GenerateFiltersAndMetricsSuggestionsResponse
            A GenerateFiltersAndMetricsSuggestionsResponse object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(GenerateFiltersAndMetricsSuggestionsResponse.from_dict(obj))


GenerateFiltersAndMetricsSuggestionsResponse._model_class = GenerateFiltersAndMetricsSuggestionsResponseModel



import json
import logging
import typing

from textwrap import dedent


if typing.TYPE_CHECKING:
    from urllib3.response import HTTPHeaderDict  # type: ignore[attr-defined]

    from snowflake.core import Root

    from .rest import RESTResponse

logger = logging.getLogger(__name__)


class SnowflakePythonError(Exception):
    """The base exception class for all Snowflake Python related Errors."""

    def __init__(self, *args: typing.Any) -> None:
        super().__init__(*args)


class _OpenAPIError(SnowflakePythonError):
    """The abstract class for all APIErrors such as TypeErrors, ValueErrors, and API Error Codes."""

    def __init__(self, *args: typing.Any) -> None:
        super().__init__(*args)
        class_name = self.__class__.__name__
        logger.debug("An exception of type %s was created, it's probably going to be raised.", class_name)
        logger.debug("This exception has the following args: %r.", args)


class _APITypeError(_OpenAPIError, TypeError):
    """Raises an exception for TypeErrors."""

    def __init__(
        self,
        msg: str,
        path_to_item: typing.Optional[list[typing.Union[str, int]]] = None,
        valid_classes: typing.Optional[tuple[type[typing.Any], ...]] = None,
        key_type: typing.Optional[bool] = None,
    ) -> None:
        """Initialize an APITypeError.

        Parameters
        __________
        msg: str
            The exception message.
        path_to_item: list, optional
            A list of keys and indices to get to the current_item. Default is None.
        valid_classes: tuple, optional
            The primitive classes that the current_item should be an instance of. Default is None.
        key_type: bool, optional
            The key_type of the current_item. True if it is a key in a dict,
            False if our value is a value in a dict or if our item is an item in a list.
            Default is None.

        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = f"{msg} at {_render_path(path_to_item)}"
        super().__init__(full_msg)


class _APIValueError(_OpenAPIError, ValueError):
    """Raises an exception for ValueErrors."""

    def __init__(self, msg: str, path_to_item: typing.Optional[list[typing.Union[str, int]]] = None) -> None:
        """Initialize an APIValueError.

        Parameters
        __________
        msg: str
            The exception message.
        path_to_item: list, optional
            The path to the exception in the received_data dict. Default is None.

        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = f"{msg} at {_render_path(path_to_item)}"
        super().__init__(full_msg)


class APIError(_OpenAPIError):
    """Raised when there is an exception with any of the API Error Codes."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
        request_id: typing.Optional[str] = None,
    ) -> None:
        self.status = status
        self.reason = reason
        self.body: typing.Optional[typing.Union[bytes, typing.Any]] = None
        self.headers: typing.Optional[HTTPHeaderDict] = None
        self.request_id = request_id
        self.root = root
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
            # Let's try to find request id if we can
            try:
                self.request_id = json.loads(self.body).get("request_id", "missing")
            except Exception:
                self.request_id = "missing"
                logger.error("Failed to extract request_id from the response body for an Error")
                logger.debug("The response body is: %r", self.body)
        super().__init__(str(self))

    def get_request_info(self) -> dict[str, typing.Any]:
        message = None
        request_id = None
        error_code = None
        if self.body:
            if isinstance(self.body, bytes):
                body = self.body.decode()
            else:
                body = self.body
            if body:
                json_body = json.loads(body)
                message = json_body.get("message", "").strip()
                request_id = json_body.get("request_id", "")
                error_code = json_body.get("error_code", "")
                if not error_code:
                    error_code = json_body.get("code", "")

        return {
            "message": message,
            "request_id": request_id,
            "error_code": error_code,
        }

    def __str__(self) -> str:
        """Provide a custom error message for the exception."""
        error_message = f"({self.status})\nReason: {self.reason}\n"
        request_info = self.get_request_info()
        if request_info["message"]:
            error_message += dedent(f"""\
            Error Message: {request_info["message"]}
            HTTP response code: {self.status}
            Request ID: {request_info["request_id"]}
            Error Code: {request_info["error_code"]}""")

        return error_message


class NotFoundError(APIError):
    """Raised when we encounter an HTTP error code of 404."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
    ) -> None:
        super().__init__(root, status, reason, http_resp)


class UnauthorizedError(APIError):
    """Raised when we encounter an HTTP error code of 401."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
    ) -> None:
        super().__init__(root, status, reason, http_resp)


class ForbiddenError(APIError):
    """Raised when we encounter an HTTP error code of 403."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
    ) -> None:
        super().__init__(root, status, reason, http_resp)


class ServerError(APIError):
    """Raised when we encounter an HTTP error code of 5NN."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
    ) -> None:
        super().__init__(root, status, reason, http_resp)


class ConflictError(APIError):
    """Raised when we encounter an HTTP error code of 409."""

    def __init__(
        self,
        root: "Root",
        status: typing.Optional[int] = None,
        reason: typing.Optional[str] = None,
        http_resp: typing.Optional["RESTResponse"] = None,
    ) -> None:
        super().__init__(root, status, reason, http_resp)


class UnexpectedResponseError(APIError):
    """Raised when spark-connect endpoint returns a response other than the expected protobuf."""

    def __init__(self, root: "Root", resp: typing.Any):
        super().__init__(root, status=200, reason="Unexpected response")
        self.body = json.dumps(resp)


def _render_path(path_to_item: typing.Optional[list[typing.Union[str, int]]]) -> str:
    """Return a string representation of a path."""
    result = ""
    if path_to_item:
        for pth in path_to_item:
            if isinstance(pth, int):
                result += f"[{pth}]"
            else:
                result += f"['{pth}']"
    return result


class InvalidActionError(SnowflakePythonError):
    """Raised when the user makes an invalid action such as invalid parameters or when the API response is invalid.

    Base class for all errors such as InvalidArgumentsError, InvalidResponseError, etc.
    """

    def __init__(self, *args: typing.Any) -> None:
        super().__init__(*args)


class InvalidResponseError(InvalidActionError):
    """Raised when the api response is invalid."""

    def __init__(self, reason: typing.Optional[str] = None) -> None:
        super().__init__(reason)
        self.reason = reason


class InvalidArgumentsError(InvalidActionError):
    """Raised when function args are invalid."""

    def __init__(self, reason: typing.Optional[str] = None) -> None:
        super().__init__(reason)
        self.reason = reason


class InvalidResultError(InvalidActionError):
    """Raised when function result is invalid."""

    def __init__(self, reason: typing.Optional[str] = None) -> None:
        super().__init__(reason)
        self.reason = reason


class InvalidOperationError(InvalidActionError):
    """Raised when user tries to make an invalid operation."""

    def __init__(self, reason: typing.Optional[str] = None) -> None:
        super().__init__(reason)
        self.reason = reason


class RetryTimeoutError(SnowflakePythonError):
    """Raised when a request times out after any amount of retries."""

    def __init__(self, reason: typing.Optional[str] = None) -> None:
        super().__init__(reason)
        self.reason = reason


class MissingModuleError(SnowflakePythonError):
    """Exception for missing modules."""

    def __init__(self, module: str, post_error_blurb: typing.Optional[str] = None) -> None:
        error_msg = f"Missing module: {module}"
        if post_error_blurb is not None:
            error_msg += f" {post_error_blurb}"
        super().__init__(error_msg)


class FileOperationError(SnowflakePythonError): ...


class FilePutError(FileOperationError): ...


class FileGetError(FileOperationError): ...


class InvalidIdentifierError(SnowflakePythonError):
    """Raised when an identifier does not meet identifier requirements."""

    def __init__(self, identifier: str) -> None:
        super().__init__(f"'{identifier}' is not a valid identifier.")



"""
Snowflake Compute Pools API.

The Snowflake Compute Pools API is a REST API that you can use to access, update, and perform common actions on Compute Pool resources.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional, Union

from pydantic import BaseModel, ConfigDict, StrictFloat, StrictInt, StrictStr


class ComputePoolInstanceFamily(BaseModel):
    """A model object representing the ComputePoolInstanceFamily resource.

    Constructs an object of type ComputePoolInstanceFamily with the provided properties.

    Parameters
    __________
    name : str, optional
        Name of the instance family.
    description : str, optional
        Description of the instance family.
    vcpu : int, optional
        Number of vCPUs.
    memory_gib : int, optional
        Memory in GiB.
    storage_gib : float, optional
        Storage in GiB.
    gpu : str, optional
        GPU model (if applicable).
    gpu_count : int, optional
        Number of GPUs.
    gpu_memory_gib : int, optional
        GPU memory in GiB (if applicable).
    current_node_usage : int, optional
        Number of nodes currently utilizing this instance family
    message : str, optional
        Additional information about the instance family.
    """

    name: Optional[StrictStr] = None

    description: Optional[StrictStr] = None

    vcpu: Optional[StrictInt] = None

    memory_gib: Optional[StrictInt] = None

    storage_gib: Optional[Union[StrictFloat, StrictInt]] = None

    gpu: Optional[StrictStr] = None

    gpu_count: Optional[StrictInt] = None

    gpu_memory_gib: Optional[StrictInt] = None

    current_node_usage: Optional[StrictInt] = None

    message: Optional[StrictStr] = None

    __properties = [
        "name",
        "description",
        "vcpu",
        "memory_gib",
        "storage_gib",
        "gpu",
        "gpu_count",
        "gpu_memory_gib",
        "current_node_usage",
        "message",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ComputePoolInstanceFamily:
        """Create an instance of ComputePoolInstanceFamily from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if gpu (nullable) is None
        if self.gpu is None:
            _dict["gpu"] = None

        # set to None if message (nullable) is None
        if self.message is None:
            _dict["message"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ComputePoolInstanceFamily:
        """Create an instance of ComputePoolInstanceFamily from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ComputePoolInstanceFamily.model_validate(obj)

        _obj = ComputePoolInstanceFamily.model_validate(
            {
                "name": obj.get("name"),
                "description": obj.get("description"),
                "vcpu": obj.get("vcpu"),
                "memory_gib": obj.get("memory_gib"),
                "storage_gib": obj.get("storage_gib"),
                "gpu": obj.get("gpu"),
                "gpu_count": obj.get("gpu_count"),
                "gpu_memory_gib": obj.get("gpu_memory_gib"),
                "current_node_usage": obj.get("current_node_usage"),
                "message": obj.get("message"),
            }
        )

        return _obj


class ComputePoolInstanceFamilyModel:
    def __init__(
        self,  # optional properties
        name: Optional[str] = None,
        description: Optional[str] = None,
        vcpu: Optional[int] = None,
        memory_gib: Optional[int] = None,
        storage_gib: Optional[float] = None,
        gpu: Optional[str] = None,
        gpu_count: Optional[int] = None,
        gpu_memory_gib: Optional[int] = None,
        current_node_usage: Optional[int] = None,
        message: Optional[str] = None,
    ):
        """A model object representing the ComputePoolInstanceFamily resource.

        Constructs an object of type ComputePoolInstanceFamily with the provided properties.

        Parameters
        __________
        name : str, optional
            Name of the instance family.
        description : str, optional
            Description of the instance family.
        vcpu : int, optional
            Number of vCPUs.
        memory_gib : int, optional
            Memory in GiB.
        storage_gib : float, optional
            Storage in GiB.
        gpu : str, optional
            GPU model (if applicable).
        gpu_count : int, optional
            Number of GPUs.
        gpu_memory_gib : int, optional
            GPU memory in GiB (if applicable).
        current_node_usage : int, optional
            Number of nodes currently utilizing this instance family
        message : str, optional
            Additional information about the instance family.
        """
        self.name = name
        self.description = description
        self.vcpu = vcpu
        self.memory_gib = memory_gib
        self.storage_gib = storage_gib
        self.gpu = gpu
        self.gpu_count = gpu_count
        self.gpu_memory_gib = gpu_memory_gib
        self.current_node_usage = current_node_usage
        self.message = message

    __properties = [
        "name",
        "description",
        "vcpu",
        "memory_gib",
        "storage_gib",
        "gpu",
        "gpu_count",
        "gpu_memory_gib",
        "current_node_usage",
        "message",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ComputePoolInstanceFamily(
            name=self.name,
            description=self.description,
            vcpu=self.vcpu,
            memory_gib=self.memory_gib,
            storage_gib=self.storage_gib,
            gpu=self.gpu,
            gpu_count=self.gpu_count,
            gpu_memory_gib=self.gpu_memory_gib,
            current_node_usage=self.current_node_usage,
            message=self.message,
        )

    @classmethod
    def _from_model(cls, model) -> ComputePoolInstanceFamilyModel:
        return ComputePoolInstanceFamilyModel(
            name=model.name,
            description=model.description,
            vcpu=model.vcpu,
            memory_gib=model.memory_gib,
            storage_gib=model.storage_gib,
            gpu=model.gpu,
            gpu_count=model.gpu_count,
            gpu_memory_gib=model.gpu_memory_gib,
            current_node_usage=model.current_node_usage,
            message=model.message,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ComputePoolInstanceFamily.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ComputePoolInstanceFamily object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ComputePoolInstanceFamilyModel:
        """Create an instance of ComputePoolInstanceFamily from a dict.

        This method constructs a ComputePoolInstanceFamily object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ComputePoolInstanceFamily
            A ComputePoolInstanceFamily object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ComputePoolInstanceFamily.from_dict(obj))


ComputePoolInstanceFamily._model_class = ComputePoolInstanceFamilyModel



"""
Snowflake API Integration API.

The Snowflake API Integration API is a REST API that you can use to access, update, and perform certain actions on API Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.api_integration._generated.models.api_hook import ApiHook, ApiHookModel


class ApiIntegration(BaseModel):
    """A model object representing the ApiIntegration resource.

    Constructs an object of type ApiIntegration with the provided properties.

    Parameters
    __________
    name : str
        Name of the API integration.
    api_hook : ApiHook

    api_allowed_prefixes : list[str]
        A comma-separated list of endpoints and resources that Snowflake can access.
    enabled : bool
        Whether the API integration is enabled.
    api_blocked_prefixes : list[str], optional
        A comma-separated list of endpoints and resources that are not allowed to be called from Snowflake.
    comment : str, optional
        Comment for the API integration.
    created_on : datetime, optional
        Date and time when the API integration was created — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    api_hook: ApiHook

    api_allowed_prefixes: List[StrictStr]

    api_blocked_prefixes: Optional[List[StrictStr]] = None

    enabled: StrictBool

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    __properties = [
        "name",
        "api_hook",
        "api_allowed_prefixes",
        "api_blocked_prefixes",
        "enabled",
        "comment",
        "created_on",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ApiIntegration:
        """Create an instance of ApiIntegration from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of api_hook
        if self.api_hook:
            _dict["api_hook"] = self.api_hook.to_dict()

        # set to None if api_blocked_prefixes (nullable) is None
        if self.api_blocked_prefixes is None:
            _dict["api_blocked_prefixes"] = None

        # set to None if comment (nullable) is None
        if self.comment is None:
            _dict["comment"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ApiIntegration:
        """Create an instance of ApiIntegration from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ApiIntegration.model_validate(obj)

        _obj = ApiIntegration.model_validate(
            {
                "name": obj.get("name"),
                "api_hook": ApiHook.from_dict(obj.get("api_hook")) if obj.get("api_hook") is not None else None,
                "api_allowed_prefixes": obj.get("api_allowed_prefixes"),
                "api_blocked_prefixes": obj.get("api_blocked_prefixes"),
                "enabled": obj.get("enabled"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
            }
        )

        return _obj


class ApiIntegrationModel:
    def __init__(
        self,
        name: str,
        api_hook: ApiHook,
        api_allowed_prefixes: list[str],
        enabled: bool,
        # optional properties
        api_blocked_prefixes: Optional[list[str]] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
    ):
        """A model object representing the ApiIntegration resource.

        Constructs an object of type ApiIntegration with the provided properties.

        Parameters
        __________
        name : str
            Name of the API integration.
        api_hook : ApiHook

        api_allowed_prefixes : list[str]
            A comma-separated list of endpoints and resources that Snowflake can access.
        enabled : bool
            Whether the API integration is enabled.
        api_blocked_prefixes : list[str], optional
            A comma-separated list of endpoints and resources that are not allowed to be called from Snowflake.
        comment : str, optional
            Comment for the API integration.
        created_on : datetime, optional
            Date and time when the API integration was created.
        """
        self.name = name
        self.api_hook = api_hook
        self.api_allowed_prefixes = api_allowed_prefixes
        self.api_blocked_prefixes = api_blocked_prefixes
        self.enabled = enabled
        self.comment = comment
        self.created_on = created_on

    __properties = [
        "name",
        "api_hook",
        "api_allowed_prefixes",
        "api_blocked_prefixes",
        "enabled",
        "comment",
        "created_on",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ApiIntegration(
            name=self.name,
            api_hook=self.api_hook._to_model() if self.api_hook is not None else None,
            api_allowed_prefixes=self.api_allowed_prefixes,
            api_blocked_prefixes=self.api_blocked_prefixes,
            enabled=self.enabled,
            comment=self.comment,
            created_on=self.created_on,
        )

    @classmethod
    def _from_model(cls, model) -> ApiIntegrationModel:
        return ApiIntegrationModel(
            name=model.name,
            api_hook=ApiHookModel._from_model(model.api_hook) if model.api_hook else None,
            api_allowed_prefixes=model.api_allowed_prefixes,
            api_blocked_prefixes=model.api_blocked_prefixes,
            enabled=model.enabled,
            comment=model.comment,
            created_on=model.created_on,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ApiIntegration.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ApiIntegration object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ApiIntegrationModel:
        """Create an instance of ApiIntegration from a dict.

        This method constructs a ApiIntegration object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ApiIntegration
            A ApiIntegration object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ApiIntegration.from_dict(obj))


ApiIntegration._model_class = ApiIntegrationModel



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class QuerySimilarityBoost(BaseModel):
    """A model object representing the QuerySimilarityBoost resource.

    Constructs an object of type QuerySimilarityBoost with the provided properties.

    Parameters
    __________
    cortex_search_service : str, optional
        Fully-qualified name of a Cortex Search Service through which to find similar previous queries.
    previous_query_column : str, optional
        Name of a TEXT column indexed in the specified previous queries search service that contains the previous query strings.
    documents_column : str, optional
        Name of an ARRAY column indexed in the specified previous queries search service that contains a list of documents that matched the corresponding previous query. Each document is represented as an OBJECT mapping all primary key columns to their corresponding values.
    """

    cortex_search_service: Optional[StrictStr] = None

    previous_query_column: Optional[StrictStr] = None

    documents_column: Optional[StrictStr] = None

    __properties = ["cortex_search_service", "previous_query_column", "documents_column"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> QuerySimilarityBoost:
        """Create an instance of QuerySimilarityBoost from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> QuerySimilarityBoost:
        """Create an instance of QuerySimilarityBoost from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return QuerySimilarityBoost.model_validate(obj)

        _obj = QuerySimilarityBoost.model_validate(
            {
                "cortex_search_service": obj.get("cortex_search_service"),
                "previous_query_column": obj.get("previous_query_column"),
                "documents_column": obj.get("documents_column"),
            }
        )

        return _obj


class QuerySimilarityBoostModel:
    def __init__(
        self,  # optional properties
        cortex_search_service: Optional[str] = None,
        previous_query_column: Optional[str] = None,
        documents_column: Optional[str] = None,
    ):
        """A model object representing the QuerySimilarityBoost resource.

        Constructs an object of type QuerySimilarityBoost with the provided properties.

        Parameters
        __________
        cortex_search_service : str, optional
            Fully-qualified name of a Cortex Search Service through which to find similar previous queries.
        previous_query_column : str, optional
            Name of a TEXT column indexed in the specified previous queries search service that contains the previous query strings.
        documents_column : str, optional
            Name of an ARRAY column indexed in the specified previous queries search service that contains a list of documents that matched the corresponding previous query. Each document is represented as an OBJECT mapping all primary key columns to their corresponding values.
        """
        self.cortex_search_service = cortex_search_service
        self.previous_query_column = previous_query_column
        self.documents_column = documents_column

    __properties = ["cortex_search_service", "previous_query_column", "documents_column"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return QuerySimilarityBoost(
            cortex_search_service=self.cortex_search_service,
            previous_query_column=self.previous_query_column,
            documents_column=self.documents_column,
        )

    @classmethod
    def _from_model(cls, model) -> QuerySimilarityBoostModel:
        return QuerySimilarityBoostModel(
            cortex_search_service=model.cortex_search_service,
            previous_query_column=model.previous_query_column,
            documents_column=model.documents_column,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a QuerySimilarityBoost.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the QuerySimilarityBoost object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> QuerySimilarityBoostModel:
        """Create an instance of QuerySimilarityBoost from a dict.

        This method constructs a QuerySimilarityBoost object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        QuerySimilarityBoost
            A QuerySimilarityBoost object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(QuerySimilarityBoost.from_dict(obj))


QuerySimilarityBoost._model_class = QuerySimilarityBoostModel



# coding: utf-8

# flake8: noqa
"""
Snowflake Catalog Integration API.

The Snowflake Catalog Integration API is a REST API that you can use to access, update, and perform certain actions on Catalog Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.catalog_integration._generated.models.catalog import Catalog
from snowflake.core.catalog_integration._generated.models.catalog_integration import CatalogIntegration
from snowflake.core.catalog_integration._generated.models.error_response import ErrorResponse
from snowflake.core.catalog_integration._generated.models.glue import Glue
from snowflake.core.catalog_integration._generated.models.o_auth import OAuth
from snowflake.core.catalog_integration._generated.models.object_store import ObjectStore
from snowflake.core.catalog_integration._generated.models.polaris import Polaris
from snowflake.core.catalog_integration._generated.models.rest_authentication import RestAuthentication
from snowflake.core.catalog_integration._generated.models.rest_config import RestConfig
from snowflake.core.catalog_integration._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.catalog_integration._generated.models.success_response import SuccessResponse
from snowflake.core.catalog_integration._generated.models.tag_assignment import TagAssignment
from snowflake.core.catalog_integration._generated.models.tag_reference import TagReference

__all__ = [
    "Catalog",
    "CatalogIntegration",
    "ErrorResponse",
    "Glue",
    "OAuth",
    "ObjectStore",
    "Polaris",
    "RestAuthentication",
    "RestConfig",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
]



from snowflake.core._internal.utils import normalize_and_unquote_name


def test_fetch(temp_stage, temp_stage_case_sensitive):
    stage = temp_stage.fetch()
    assert stage.name.upper() == temp_stage.name.upper()

    stage = temp_stage_case_sensitive.fetch()
    assert stage.name == normalize_and_unquote_name(temp_stage_case_sensitive.name)
    assert stage.kind == "PERMANENT"
    assert stage.url is not None
    assert stage.storage_integration is None
    assert stage.comment == "created by temp_stage"
    assert stage.created_on is not None
    assert stage.owner_role_type is not None
    assert stage.directory_table is not None
    assert stage.has_credentials is not None
    assert stage.has_encryption_key is not None



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import copy
import json

from collections.abc import Iterable, Iterator
from datetime import timedelta

import pytest

from snowflake.core import CreateMode
from snowflake.core.schema import Schema
from snowflake.core.task import Cron, Task, TaskCollection

from ..utils import random_object_name


task_name1 = random_object_name()
task_name2 = random_object_name()
task_name3 = random_object_name()
task_name4 = random_object_name()
name_list = [task_name1, task_name2, task_name3, task_name4]


@pytest.fixture(scope="module")
def tasks(db_parameters, database) -> Iterator[TaskCollection]:
    warehouse_name = db_parameters["warehouse"]
    schema = database.schemas.create(Schema(name=(random_object_name())))
    tasks = [
        Task(
            name=task_name2,
            definition="select current_version()",
            suspend_task_after_num_failures=10,
            schedule=timedelta(minutes=10),
            allow_overlapping_execution=True,
        ),
        Task(
            name=task_name3,
            definition="select current_version()",
            user_task_managed_initial_warehouse_size="xsmall",
            target_completion_interval=timedelta(minutes=5),
            serverless_task_min_statement_size="xsmall",
            serverless_task_max_statement_size="small",
            schedule=Cron("0 9-17 * * SUN", "America/Los_Angeles"),
        ),
        Task(
            name=task_name1,
            definition="select current_version()",
            warehouse=warehouse_name,
            comment="test_task",
            predecessors=[task_name2, task_name3],
        ),
        Task(
            name=task_name4,
            definition="select current_version()",
            warehouse=warehouse_name,
            comment="test_task",
            finalize=task_name2,
        ),
    ]
    task_collection = schema.tasks
    for task in tasks:
        task_collection.create(task, mode=CreateMode.or_replace)
    try:
        yield task_collection
    finally:
        schema.drop()


def test_basic(tasks):
    result = _info_list_to_dict(tasks.iter())
    for t in [task_name1, task_name2, task_name3, task_name4]:
        assert t.upper() in result
        res = result[t.upper()]
        task = tasks[t].fetch()
        assert res.created_on == task.created_on
        assert res.name == task.name
        assert task.id == res.id
        assert task.database_name == res.database_name
        assert task.schema_name == res.schema_name
        assert task.owner == res.owner
        assert task.definition == res.definition
        assert task.warehouse == res.warehouse
        assert task.comment == res.comment
        assert task.state == res.state
        assert task.condition == res.condition
        assert task.error_integration == res.error_integration
        assert task.last_committed_on == res.last_committed_on
        assert task.last_suspended_on == res.last_suspended_on


@pytest.mark.min_sf_ver("8.27.0")
def test_finalize_support(tasks):
    database = tasks.database
    schema = tasks.schema
    result = _info_list_to_dict(tasks.iter())
    # assert finalize
    res = result[task_name4.upper()]
    assert res.finalize == f"{task_name2.upper()}"
    for t in [task_name1, task_name2, task_name3]:
        res = result[t.upper()]
        assert res.finalize is None

    # assert task relations
    res = result[task_name4.upper()]
    task_relations = json.loads(res.task_relations)
    assert task_relations["FinalizedRootTask"] == f"{database.name.upper()}.{schema.name.upper()}.{task_name2.upper()}"
    assert task_relations["Predecessors"] == []
    assert task_relations.get("FinalizerTask", None) is None

    res = result[task_name2.upper()]
    task_relations = json.loads(res.task_relations)
    assert task_relations["FinalizerTask"] == f"{database.name.upper()}.{schema.name.upper()}.{task_name4.upper()}"
    assert task_relations["Predecessors"] == []
    assert task_relations.get("FinalizedRootTask", None) is None

    res = result[task_name3.upper()]
    task_relations = json.loads(res.task_relations)
    assert task_relations.get("FinalizerTask", None) is None
    assert task_relations.get("FinalizedRootTask", None) is None
    assert task_relations["Predecessors"] == []

    res = result[task_name1.upper()]
    task_relations = json.loads(res.task_relations)
    assert task_relations.get("FinalizerTask", None) is None
    assert task_relations.get("FinalizedRootTask", None) is None
    assert set(task_relations["Predecessors"]) == set(
        [
            f"{database.name.upper()}.{schema.name.upper()}.{task_name2.upper()}",
            f"{database.name.upper()}.{schema.name.upper()}.{task_name3.upper()}",
        ]
    )


@pytest.mark.min_sf_ver("9.4.0")
def test_serverless_attributes(tasks):
    result = _info_list_to_dict(tasks.iter())
    for t in [task_name1, task_name2, task_name3, task_name4]:
        assert t.upper() in result
        res = result[t.upper()]
        task = tasks[t].fetch()
        assert task.user_task_managed_initial_warehouse_size == res.user_task_managed_initial_warehouse_size
        assert task.target_completion_interval == res.target_completion_interval
        assert task.serverless_task_min_statement_size == res.serverless_task_min_statement_size
        assert task.serverless_task_max_statement_size == res.serverless_task_max_statement_size


def test_pattern(tasks):
    result = _info_list_to_dict(tasks.iter(like=task_name1))
    assert task_name1.upper() in result
    assert len(result) == 1
    result = _info_list_to_dict(tasks.iter(like=random_object_name()))
    assert len(result) == 0
    result = _info_list_to_dict(tasks.iter(like="test_object%"))
    assert task_name1.upper() in result
    assert task_name2.upper() in result
    assert task_name3.upper() in result


@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
def test_like(tasks):
    result = tasks.iter(like="")
    assert len(list(result)) == 0


def test_limit_from(tasks):
    result = tasks.iter()
    assert len(list(result)) >= 4

    result = tasks.iter(limit=3)
    assert len(list(result)) == 3

    lex_order_names = copy.deepcopy(name_list)
    lex_order_names.sort()
    # use the second last task_name as 'from_name'
    result = _info_list_to_dict(tasks.iter(limit=3, from_name=lex_order_names[-2][:-1].upper()))
    assert len(result) >= 2
    assert lex_order_names[-2].upper() in result
    assert lex_order_names[-1].upper() in result

    # test case-sensitive
    result = _info_list_to_dict(tasks.iter(limit=3, from_name=lex_order_names[-2][:-1]))
    assert len(result) == 0


def _info_list_to_dict(info_list: Iterable[Task]) -> dict[str, Task]:
    result = {}
    for info in info_list:
        result[info.name] = info
    return result



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.warehouse._generated.api.warehouse_api import WarehouseApi

__all__ = [
    "WarehouseApi",
]



from typing import Optional

import pytest

from snowflake.core import CreateMode
from snowflake.core.tag import TagResource, TagValue
from snowflake.core.user_defined_function import (
    ReturnDataType,
    SQLFunction,
    UserDefinedFunction,
    UserDefinedFunctionCollection,
)

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


class TestUserDefinedFunctionTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, user_defined_functions: UserDefinedFunctionCollection):
        self.user_defined_functions = user_defined_functions
        udf_name = random_string(6, "udf_for_tag_tests_")
        udf = UserDefinedFunction(
            name=udf_name,
            arguments=[],
            return_type=ReturnDataType(datatype="VARCHAR"),
            language_config=SQLFunction(),
            body="SELECT 'test'",
        )
        self.udf_res = user_defined_functions.create(udf, mode=CreateMode.if_not_exists)
        try:
            yield
        finally:
            self.udf_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "FUNCTION"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.user_defined_functions[
            self._normalize_resource_name(resource_name) or self.udf_res.name_with_args
        ].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.user_defined_functions[
            self._normalize_resource_name(resource_name) or self.udf_res.name_with_args
        ].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.user_defined_functions[
            self._normalize_resource_name(resource_name) or self.udf_res.name_with_args
        ].get_tags(with_lineage)

    @staticmethod
    def _normalize_resource_name(name: str | None) -> str | None:
        return f"{name}()" if name and not name.endswith(")") else name



"""
Snowflake View API.

The Snowflake View API is a REST API that you can use to access, update, and perform certain actions on View resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr, field_validator

from snowflake.core.view._generated.models.view_column import ViewColumn, ViewColumnModel


class View(BaseModel):
    """A model object representing the View resource.

    Constructs an object of type View with the provided properties.

    Parameters
    __________
    name : str
        Name of the view
    columns : list[ViewColumn]
        The columns of the view
    query : str
        Query used to create the view
    secure : bool, optional
        Whether or not this view is secure
    kind : str, optional
        Kind of the view, permanent (default) or temporary
    recursive : bool, optional
        Whether or not this view can refer to itself using recursive syntax withot requiring a CTE (common table expression)
    comment : str, optional
        user comment associated to an object in the dictionary
    created_on : datetime, optional
        Date and time when the view was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the view is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the view is stored — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the view — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the view — **Read-only:** *any user-provided value will be ignored.*
    """

    name: StrictStr

    secure: Optional[StrictBool] = None

    kind: Optional[StrictStr] = None

    recursive: Optional[StrictBool] = None

    columns: List[ViewColumn]

    comment: Optional[StrictStr] = None

    query: StrictStr

    created_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    __properties = [
        "name",
        "secure",
        "kind",
        "recursive",
        "columns",
        "comment",
        "query",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
    ]

    @field_validator("kind")
    def kind_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("PERMANENT", "TEMPORARY"):
            raise ValueError("must validate the enum values ('PERMANENT','TEMPORARY')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> View:
        """Create an instance of View from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "owner",
                    "owner_role_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict["columns"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> View:
        """Create an instance of View from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return View.model_validate(obj)

        _obj = View.model_validate(
            {
                "name": obj.get("name"),
                "secure": obj.get("secure"),
                "kind": obj.get("kind"),
                "recursive": obj.get("recursive"),
                "columns": [ViewColumn.from_dict(_item) for _item in obj.get("columns")]
                if obj.get("columns") is not None
                else None,
                "comment": obj.get("comment"),
                "query": obj.get("query"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
            }
        )

        return _obj


class ViewModel:
    def __init__(
        self,
        name: str,
        columns: list[ViewColumn],
        query: str,
        # optional properties
        secure: Optional[bool] = None,
        kind: Optional[str] = None,
        recursive: Optional[bool] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
    ):
        """A model object representing the View resource.

        Constructs an object of type View with the provided properties.

        Parameters
        __________
        name : str
            Name of the view
        columns : list[ViewColumn]
            The columns of the view
        query : str
            Query used to create the view
        secure : bool, optional
            Whether or not this view is secure
        kind : str, optional
            Kind of the view, permanent (default) or temporary
        recursive : bool, optional
            Whether or not this view can refer to itself using recursive syntax withot requiring a CTE (common table expression)
        comment : str, optional
            user comment associated to an object in the dictionary
        created_on : datetime, optional
            Date and time when the view was created.
        database_name : str, optional
            Database in which the view is stored
        schema_name : str, optional
            Schema in which the view is stored
        owner : str, optional
            Role that owns the view
        owner_role_type : str, optional
            The type of role that owns the view
        """
        self.name = name
        self.secure = secure
        self.kind = kind
        self.recursive = recursive
        self.columns = columns
        self.comment = comment
        self.query = query
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner
        self.owner_role_type = owner_role_type

    __properties = [
        "name",
        "secure",
        "kind",
        "recursive",
        "columns",
        "comment",
        "query",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return View(
            name=self.name,
            secure=self.secure,
            kind=self.kind,
            recursive=self.recursive,
            columns=[x._to_model() for x in self.columns] if self.columns is not None else None,
            comment=self.comment,
            query=self.query,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
        )

    @classmethod
    def _from_model(cls, model) -> ViewModel:
        return ViewModel(
            name=model.name,
            secure=model.secure,
            kind=model.kind,
            recursive=model.recursive,
            columns=[ViewColumnModel._from_model(x) for x in model.columns] if model.columns else None,
            comment=model.comment,
            query=model.query,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a View.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the View object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ViewModel:
        """Create an instance of View from a dict.

        This method constructs a View object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        View
            A View object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(View.from_dict(obj))


View._model_class = ViewModel



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.cortex.inference_service import (
    CompleteRequest,
    CompleteRequestMessagesInner,
    CortexInferenceService,
)

from ...utils import BASE_URL, extra_params


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def inference_service(fake_root):
    return CortexInferenceService(fake_root)


def test_complete(fake_root, inference_service):
    args = (fake_root, "POST", BASE_URL + "/cortex/inference:complete")
    kwargs = extra_params(
        body={
            "model": "my_model",
            "messages": [{"role": "user", "content": "xyz"}],
            "top_p": 1.0,
            "max_tokens": 4096,
            "stream": True,
            "anthropic": None,
            "openai": None,
            "temperature": None,
            "max_output_tokens": None,
            "response_format": None,
            "guardrails": None,
            "tool_choice": None,
            "provisioned_throughput_id": None,
        },
        _preload_content=False,
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = inference_service.complete_async(
            CompleteRequest(model="my_model", messages=[CompleteRequestMessagesInner(content="xyz")])
        )
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.stage import FileTransferMaterial, PresignedUrlRequest, Stage, StageResource

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def stages(schema):
    return schema.stages


@pytest.fixture
def stage(stages):
    return stages["my_stage"]


def test_create_stage(fake_root, stages):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/stages")
    kwargs = extra_params(query_params=[], body={"name": "my_stage", "kind": "PERMANENT"})

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stage_res = stages.create(Stage(name="my_stage"))
        assert isinstance(stage_res, StageResource)
        assert stage_res.name == "my_stage"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = stages.create_async(Stage(name="my_stage"))
        assert isinstance(op, PollingOperation)
        stage_res = op.result()
        assert stage_res.name == "my_stage"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_stage(fake_root, stages):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/stages")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        stages.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = stages.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_stage(fake_root, stage):
    from snowflake.core.stage._generated.models import Stage as StageModel

    model = StageModel(name="my_stage")
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/stages/my_stage")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        stage.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = stage.fetch_async()
        assert isinstance(op, PollingOperation)
        stage = op.result()
        assert stage.to_dict() == Stage(name="my_stage").to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_stage(fake_root, stage):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/stages/my_stage")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        stage.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = stage.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_list_files(fake_root, stage):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/stages/my_stage/files")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        stage.list_files()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = stage.list_files_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_presigned_url(fake_root, stage):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/stages/my_stage/files/my_file:presigned-url",
    )
    kwargs = extra_params(body={})
    mocked_response = FileTransferMaterial(presigned_url="localhost:40000/my_file").to_json()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(mocked_response)
        presigned_url = stage.get_presigned_url("my_file", PresignedUrlRequest()).presigned_url
        assert presigned_url == "localhost:40000/my_file"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(mocked_response)
        op = stage.get_presigned_url_async("my_file", PresignedUrlRequest())
        assert isinstance(op, PollingOperation)
        presigned_url = op.result().presigned_url
        assert presigned_url == "localhost:40000/my_file"
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictStr


class FetchServiceStatus200Response(BaseModel):
    """A model object representing the FetchServiceStatus200Response resource.

    Constructs an object of type FetchServiceStatus200Response with the provided properties.

    Parameters
    __________
    systemget_service_status : str, optional
    """

    systemget_service_status: Optional[StrictStr] = Field(default=None, alias="system$get_service_status")

    __properties = ["system$get_service_status"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> FetchServiceStatus200Response:
        """Create an instance of FetchServiceStatus200Response from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> FetchServiceStatus200Response:
        """Create an instance of FetchServiceStatus200Response from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return FetchServiceStatus200Response.model_validate(obj)

        _obj = FetchServiceStatus200Response.model_validate(
            {
                "systemget_service_status": obj.get("system$get_service_status"),
            }
        )

        return _obj


class FetchServiceStatus200ResponseModel:
    def __init__(
        self,  # optional properties
        systemget_service_status: Optional[str] = None,
    ):
        """A model object representing the FetchServiceStatus200Response resource.

        Constructs an object of type FetchServiceStatus200Response with the provided properties.

        Parameters
        __________
        systemget_service_status : str, optional
        """
        self.systemget_service_status = systemget_service_status

    __properties = ["system$get_service_status"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return FetchServiceStatus200Response(
            systemget_service_status=self.systemget_service_status,
        )

    @classmethod
    def _from_model(cls, model) -> FetchServiceStatus200ResponseModel:
        return FetchServiceStatus200ResponseModel(
            systemget_service_status=model.systemget_service_status,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a FetchServiceStatus200Response.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the FetchServiceStatus200Response object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> FetchServiceStatus200ResponseModel:
        """Create an instance of FetchServiceStatus200Response from a dict.

        This method constructs a FetchServiceStatus200Response object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        FetchServiceStatus200Response
            A FetchServiceStatus200Response object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(FetchServiceStatus200Response.from_dict(obj))


FetchServiceStatus200Response._model_class = FetchServiceStatus200ResponseModel



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import json

from collections.abc import Generator
from datetime import timedelta

import pytest

from snowflake.connector import DictCursor
from snowflake.core.exceptions import NotFoundError

from ..utils import random_object_name


task_name1 = random_object_name()
task_name2 = random_object_name()
task_name3 = random_object_name()
task_name4 = random_object_name()


@pytest.fixture(scope="module", autouse=True)
def setup(tasks, connection) -> Generator[None, None, None]:
    with connection.cursor() as cur:
        warehouse_name: str = cur.execute("select current_warehouse();").fetchone()[0]
        create_task2 = (
            f"create or replace task {task_name2} "
            "ALLOW_OVERLAPPING_EXECUTION = true SUSPEND_TASK_AFTER_NUM_FAILURES = 10 "
            "TIMEZONE='America/Los_Angeles' SNOWPARK_REQUEST_TIMEOUT_IN_SECONDS = 100 QUERY_RESULT_FORMAT='ARROW'"
            "schedule = '10 minute' as select current_version()"
        )
        cur.execute(create_task2).fetchone()
        create_task3 = (
            f"create or replace task {task_name3} "
            "user_task_managed_initial_warehouse_size = 'xsmall' "
            "target_completion_interval = '5 MINUTE' "
            "serverless_task_min_statement_size = 'xsmall' "
            "serverless_task_max_statement_size = 'small' "
            "SCHEDULE = 'USING CRON 0 9-17 * * SUN America/Los_Angeles' as select current_version()"
        )
        cur.execute(create_task3).fetchone()
        create_task1 = (
            f"create or replace task {task_name1} "
            f"warehouse = {warehouse_name} "
            "comment = 'test_task' "
            f"after {task_name2}, {task_name3} "
            "as select current_version()"
        )
        cur.execute(create_task1).fetchone()
        create_task4 = (
            f"create or replace task {task_name4} "
            f"warehouse = {warehouse_name} "
            "comment = 'test_task' "
            f"finalize = {task_name2} "
            "as select current_version()"
        )
        cur.execute(create_task4).fetchone()
        yield
        drop_task1 = f"drop task if exists {task_name1}"
        cur.execute(drop_task1).fetchone()
        drop_task2 = f"drop task if exists {task_name2}"
        cur.execute(drop_task2).fetchone()
        drop_task3 = f"drop task if exists {task_name2}"
        cur.execute(drop_task3).fetchone()
        drop_task4 = f"drop task if exists {task_name4}"
        cur.execute(drop_task4).fetchone()


@pytest.mark.snowpark
def test_load_task_basic(tasks, session):
    from snowflake.snowpark._internal.utils import parse_table_name

    task = tasks[task_name1].fetch()
    result = session._conn._conn.cursor(DictCursor).execute(f"describe task {task_name1}")
    for res in result:
        assert res["created_on"] == task.created_on
        assert res["name"] == task.name
        assert task.name == task_name1.upper()
        assert task.id == res["id"]
        assert task.database_name == res["database_name"]
        assert task.schema_name == res["schema_name"]
        assert task.owner == res["owner"]
        assert task.definition == res["definition"]
        assert task.warehouse == res["warehouse"]
        assert task.comment == res["comment"]
        assert task.comment == "test_task"
        assert task.state == res["state"]
        assert task.condition == res["condition"]
        assert task.error_integration is None
        assert task.last_committed_on == res["last_committed_on"]
        assert task.last_suspended_on == res["last_suspended_on"]
        # check predecessors
        parent_ids: list[str] = sorted(map(lambda name: parse_table_name(name)[-1], task.predecessors))
        expected = sorted([tasks[task_name2].fetch().name, tasks[task_name3].fetch().name])
        assert expected == parent_ids


def test_finalize_support(tasks, session):
    for t in [task_name1, task_name2, task_name3, task_name4]:
        task = tasks[t].fetch()
        result = session._conn._conn.cursor(DictCursor).execute(f"describe as resource task {t}")
        for res in result:
            task_relations_result = json.loads(json.loads(res["As Resource"])["task_relations"])
            task_relations_fetch = json.loads(task.task_relations)
            assert task_relations_result == task_relations_fetch


def test_load_task_allow_overlapping_execution(tasks):
    task = tasks[task_name2].fetch()
    assert task.allow_overlapping_execution


def test_load_task_user_task_managed_initial_warehouse_size(tasks):
    task = tasks[task_name3]
    assert task.fetch().user_task_managed_initial_warehouse_size == "XSMALL"


@pytest.mark.min_sf_ver("9.4.0")
def test_load_task_target_completion_interval(tasks):
    task = tasks[task_name3]
    assert task.fetch().target_completion_interval == timedelta(minutes=5)


@pytest.mark.min_sf_ver("9.4.0")
def test_load_task_serverless_task_min_statement_size(tasks):
    task = tasks[task_name3]
    assert task.fetch().serverless_task_min_statement_size == "XSMALL"


@pytest.mark.min_sf_ver("9.4.0")
def test_load_task_serverless_task_max_statement_size(tasks):
    task = tasks[task_name3]
    assert task.fetch().serverless_task_max_statement_size == "SMALL"


def test_show_parameters(tasks):
    task = tasks[task_name2]
    task_payload = task.fetch()
    local_parameters = task_payload.session_parameters
    assert local_parameters["TIMEZONE"] == "America/Los_Angeles"
    assert local_parameters["SNOWPARK_REQUEST_TIMEOUT_IN_SECONDS"] == 100
    assert local_parameters["QUERY_RESULT_FORMAT"] == "ARROW"


def test_load_task_failure(tasks):
    # lazy, no error
    task = tasks["task_not_exists"]

    with pytest.raises(NotFoundError):
        task.fetch()


def test_load_task_schedule(tasks):
    task2 = tasks[task_name2].fetch()
    assert task2.schedule == timedelta(minutes=10)
    task3 = tasks[task_name3].fetch()
    assert task3.schedule.expr == "0 9-17 * * SUN"
    assert task3.schedule.timezone == "America/Los_Angeles"



"""
Snowflake Iceberg Table API.

The Snowflake Iceberg Table API is a REST API that you can use to access, update, and perform certain actions on Iceberg Table resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr


class IcebergTableFromIcebergFiles(BaseModel):
    """A model object representing the IcebergTableFromIcebergFiles resource.

    Constructs an object of type IcebergTableFromIcebergFiles with the provided properties.

    Parameters
    __________
    name : str
        Specifies the name for the table, must be unique for the schema in which the table is created
    metadata_file_path : str
        Specifies the relative path of the Iceberg metadata file to use for column definitions.
    external_volume : str, optional
        Specifies the name of the external volume to use for the table
    replace_invalid_characters : bool, optional
        Specifies whether to replace invalid characters in the column names
    catalog : str, optional
        Specifies the name of the catalog integration to use for the table
    comment : str, optional
        Specifies a comment for the table
    """

    name: StrictStr

    external_volume: Optional[StrictStr] = None

    replace_invalid_characters: Optional[StrictBool] = None

    metadata_file_path: StrictStr

    catalog: Optional[StrictStr] = None

    comment: Optional[StrictStr] = None

    __properties = ["name", "external_volume", "replace_invalid_characters", "metadata_file_path", "catalog", "comment"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> IcebergTableFromIcebergFiles:
        """Create an instance of IcebergTableFromIcebergFiles from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> IcebergTableFromIcebergFiles:
        """Create an instance of IcebergTableFromIcebergFiles from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return IcebergTableFromIcebergFiles.model_validate(obj)

        _obj = IcebergTableFromIcebergFiles.model_validate(
            {
                "name": obj.get("name"),
                "external_volume": obj.get("external_volume"),
                "replace_invalid_characters": obj.get("replace_invalid_characters"),
                "metadata_file_path": obj.get("metadata_file_path"),
                "catalog": obj.get("catalog"),
                "comment": obj.get("comment"),
            }
        )

        return _obj


class IcebergTableFromIcebergFilesModel:
    def __init__(
        self,
        name: str,
        metadata_file_path: str,
        # optional properties
        external_volume: Optional[str] = None,
        replace_invalid_characters: Optional[bool] = None,
        catalog: Optional[str] = None,
        comment: Optional[str] = None,
    ):
        """A model object representing the IcebergTableFromIcebergFiles resource.

        Constructs an object of type IcebergTableFromIcebergFiles with the provided properties.

        Parameters
        __________
        name : str
            Specifies the name for the table, must be unique for the schema in which the table is created
        metadata_file_path : str
            Specifies the relative path of the Iceberg metadata file to use for column definitions.
        external_volume : str, optional
            Specifies the name of the external volume to use for the table
        replace_invalid_characters : bool, optional
            Specifies whether to replace invalid characters in the column names
        catalog : str, optional
            Specifies the name of the catalog integration to use for the table
        comment : str, optional
            Specifies a comment for the table
        """
        self.name = name
        self.external_volume = external_volume
        self.replace_invalid_characters = replace_invalid_characters
        self.metadata_file_path = metadata_file_path
        self.catalog = catalog
        self.comment = comment

    __properties = ["name", "external_volume", "replace_invalid_characters", "metadata_file_path", "catalog", "comment"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return IcebergTableFromIcebergFiles(
            name=self.name,
            external_volume=self.external_volume,
            replace_invalid_characters=self.replace_invalid_characters,
            metadata_file_path=self.metadata_file_path,
            catalog=self.catalog,
            comment=self.comment,
        )

    @classmethod
    def _from_model(cls, model) -> IcebergTableFromIcebergFilesModel:
        return IcebergTableFromIcebergFilesModel(
            name=model.name,
            external_volume=model.external_volume,
            replace_invalid_characters=model.replace_invalid_characters,
            metadata_file_path=model.metadata_file_path,
            catalog=model.catalog,
            comment=model.comment,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a IcebergTableFromIcebergFiles.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the IcebergTableFromIcebergFiles object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> IcebergTableFromIcebergFilesModel:
        """Create an instance of IcebergTableFromIcebergFiles from a dict.

        This method constructs a IcebergTableFromIcebergFiles object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        IcebergTableFromIcebergFiles
            A IcebergTableFromIcebergFiles object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(IcebergTableFromIcebergFiles.from_dict(obj))


IcebergTableFromIcebergFiles._model_class = IcebergTableFromIcebergFilesModel



# coding: utf-8

# flake8: noqa

"""
Snowflake Iceberg Table API.

The Snowflake Iceberg Table API is a REST API that you can use to access, update, and perform certain actions on Iceberg Table resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.iceberg_table._generated.api.iceberg_table_api import IcebergTableApi

# import models into sdk package
from snowflake.core.iceberg_table._generated.models.constraint import Constraint
from snowflake.core.iceberg_table._generated.models.convert_to_managed_iceberg_table_request import (
    ConvertToManagedIcebergTableRequest,
)
from snowflake.core.iceberg_table._generated.models.error_response import ErrorResponse
from snowflake.core.iceberg_table._generated.models.foreign_key import ForeignKey
from snowflake.core.iceberg_table._generated.models.iceberg_table import IcebergTable
from snowflake.core.iceberg_table._generated.models.iceberg_table_as_select import IcebergTableAsSelect
from snowflake.core.iceberg_table._generated.models.iceberg_table_clone import IcebergTableClone
from snowflake.core.iceberg_table._generated.models.iceberg_table_column import IcebergTableColumn
from snowflake.core.iceberg_table._generated.models.iceberg_table_from_aws_glue_catalog import (
    IcebergTableFromAWSGlueCatalog,
)
from snowflake.core.iceberg_table._generated.models.iceberg_table_from_delta import IcebergTableFromDelta
from snowflake.core.iceberg_table._generated.models.iceberg_table_from_iceberg_files import IcebergTableFromIcebergFiles
from snowflake.core.iceberg_table._generated.models.iceberg_table_from_iceberg_rest import IcebergTableFromIcebergRest
from snowflake.core.iceberg_table._generated.models.iceberg_table_like import IcebergTableLike
from snowflake.core.iceberg_table._generated.models.point_of_time import PointOfTime
from snowflake.core.iceberg_table._generated.models.point_of_time_offset import PointOfTimeOffset
from snowflake.core.iceberg_table._generated.models.point_of_time_statement import PointOfTimeStatement
from snowflake.core.iceberg_table._generated.models.point_of_time_timestamp import PointOfTimeTimestamp
from snowflake.core.iceberg_table._generated.models.primary_key import PrimaryKey
from snowflake.core.iceberg_table._generated.models.refresh_iceberg_table_request import RefreshIcebergTableRequest
from snowflake.core.iceberg_table._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.iceberg_table._generated.models.success_response import SuccessResponse
from snowflake.core.iceberg_table._generated.models.tag_assignment import TagAssignment
from snowflake.core.iceberg_table._generated.models.tag_reference import TagReference
from snowflake.core.iceberg_table._generated.models.unique_key import UniqueKey



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#
from io import BytesIO
from textwrap import dedent

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.service import Service, ServiceSpecStageFile
from tests.utils import random_string


pytestmark = [pytest.mark.skip_gov]


def test_drop(services, session, imagerepo, shared_compute_pool):
    try:
        service_name = random_string(5, "test_service_")
        stage_name = random_string(5, "test_stage_")

        session.sql(f"create temp stage {stage_name};").collect()
        spec_file = "spec.yaml"
        stage_file = f"@{stage_name}"
        spec = f"{stage_file}/{spec_file}"
        session.file.put_stream(
            BytesIO(
                dedent(f"""\
                    spec:
                      containers:
                      - name: hello-world
                        image: {imagerepo}/hello-world:latest
                """).encode()
            ),
            spec,
        )
        test_service = Service(
            name=service_name,
            compute_pool=shared_compute_pool,
            spec=ServiceSpecStageFile(stage=stage_name, spec_file=spec_file),
            min_instances=1,
            max_instances=1,
        )
        s = services.create(test_service)
        s.drop()
        with pytest.raises(NotFoundError):
            # TODO: HTTP response body: {"description": "list index out of range", "error_details": null}
            #  Looks wrong
            s.fetch()

        s = services.create(test_service)
        s.drop()
    finally:
        services[service_name].drop(if_exists=True)



from concurrent.futures import Future
from unittest import mock

import pytest

import snowflake.core._thread_pool as thread_pool

from snowflake.core._generated import ApiClient
from snowflake.core._internal.snowapi_parameters import SnowApiParameter, SnowApiParameters


@pytest.fixture(autouse=True)
def reset_pool():
    thread_pool.THREAD_POOL.reset()


def test_async_api_calls_are_submitted_to_the_pool(fake_root, event):
    fake_root.parameters.return_value = SnowApiParameters({SnowApiParameter.MAX_THREADS: "1"})
    api_client = ApiClient(fake_root)
    with mock.patch("snowflake.core._generated.api_client.ApiClient.request") as mocked_request:
        mocked_request.side_effect = lambda *args, **kwargs: event.wait()
        first_call = api_client.call_api(fake_root, "/api/v2/databases", "GET", async_req=True)
        second_call = api_client.call_api(fake_root, "/api/v2/databases", "GET", async_req=True)

    assert isinstance(first_call, Future)
    assert first_call.running()
    assert not first_call.done()

    assert isinstance(second_call, Future)
    assert not second_call.running()
    assert not second_call.done()



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from pydantic import StrictStr

from snowflake.core import PollingOperation
from snowflake.core._common import AccountObjectCollectionParent, CreateMode, ObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._internal.utils import deprecated
from snowflake.core._operation import PollingOperations
from snowflake.core._utils import (
    tag_assignment_to_tag_tuple,
    tag_resource_to_tag_reference,
    tag_tuple_to_tag_assignment,
)
from snowflake.core.tag import TagResource, TagValue
from snowflake.core.user._generated import TagAssignment, TagReference
from snowflake.core.user._generated.api import UserApi
from snowflake.core.user._generated.models.grant import Grant
from snowflake.core.user._generated.models.securable import Securable
from snowflake.core.user._generated.models.user import UserModel as User


if TYPE_CHECKING:
    from snowflake.core import Root


class UserCollection(AccountObjectCollectionParent["UserResource"]):
    """Represents the collection operations on the Snowflake User resource.

    With this collection, you can create, iterate through, and search for users that you have access to in the
    current context.

    Examples
    ________
    Creating a user instance:

    >>> sample_user = User(name="test_user")
    >>> root.users.create(sample_user)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, ref_class=UserResource)
        self._api = UserApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(self, user: User, *, mode: CreateMode = CreateMode.error_if_exists) -> "UserResource":
        """
        Create a user in Snowflake account.

        Parameters
        __________
        user: User
            The ``User`` object, together with the ``User``'s properties:
            name; password, login_name, display_name, first_name, middle_name, last_name,
            email, must_change_password, disabled, days_to_expiry, mins_to_unlock, default_warehouse,
            default_namespace, default_role, default_secondary_roles, mins_to_bypass_mfa, rsa_public_key,
            rsa_public_key_fp, rsa_public_key_2, rsa_public_key_2_fp, comment, type,
            enable_unredacted_query_syntax_error, network_policy are optional
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the user already exists in Snowflake.  Equivalent to SQL ``create user <name> ...``.

            ``CreateMode.or_replace``: Replace if the user already exists in Snowflake. Equivalent to SQL
            ``create or replace user <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the user already exists in Snowflake.
            Equivalent to SQL ``create user <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a user, replacing any existing user with the same name:

        >>> sample_user = User(name="test_user")
        >>> root.users.create(sample_user, mode=CreateMode.or_replace)
        """
        create_mode = CreateMode[mode].value
        self._api.create_user(user._to_model(), StrictStr(create_mode), async_req=False)
        return self[user.name]

    @api_telemetry
    def create_async(
        self, user: User, *, mode: CreateMode = CreateMode.error_if_exists
    ) -> PollingOperation["UserResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        create_mode = CreateMode[mode].value
        future = self._api.create_user(user._to_model(), StrictStr(create_mode), async_req=True)
        return PollingOperation(future, lambda _: self[user.name])

    @api_telemetry
    def iter(
        self,
        like: Optional[str] = None,
        limit: Optional[int] = None,
        starts_with: Optional[str] = None,
        from_name: Optional[str] = None,
    ) -> Iterator[User]:
        users = self._api.list_users(
            StrictStr(like) if like else None,
            StrictStr(starts_with) if starts_with else None,
            limit,
            from_name=from_name,
            async_req=False,
        )
        """Iterate through ``User`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear
            at the beginning of the object name. Uses case-sensitive pattern matching.
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches
            the specified string. This is case-sensitive and does not have to be the full name.

        Examples
        ________
        Showing all users that you have access to see:

        >>> users = user_collection.iter()

        Showing information of the exact user you want to see:

        >>> users = user_collection.iter(like="your-user-name")

        Showing users starting with 'your-user-name-':

        >>> users = user_collection.iter(like="your-user-name-%")
        >>> users = user_collection.iter(starts_with="your-user-name")

        Using a for loop to retrieve information from iterator:

        >>> for user in users:
        ...     print(user.name, user.display_name)
        """
        return map(User._from_model, iter(users))

    @api_telemetry
    def iter_async(
        self,
        like: Optional[str] = None,
        limit: Optional[int] = None,
        starts_with: Optional[str] = None,
        from_name: Optional[str] = None,
    ) -> PollingOperation[Iterator[User]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_users(
            StrictStr(like) if like else None,
            StrictStr(starts_with) if starts_with else None,
            limit,
            from_name=from_name,
            async_req=True,
        )
        return PollingOperation(future, lambda rest_models: map(User._from_model, iter(rest_models)))


class UserResource(ObjectReferenceMixin[UserCollection]):
    """Represents a reference to a Snowflake user.

    With this user reference, you can fetch information about a user, as well as perform certain
    actions on it.
    """

    def __init__(self, name: str, collection: UserCollection) -> None:
        self.name = name
        self.collection = collection

    @api_telemetry
    def create_or_alter(self, user: User) -> None:
        """Create a user in Snowflake or alter one if it already exists.

        Parameters
        __________
        user: User
            The ``User`` object, together with the ``User``'s properties:
            name; password, login_name, display_name, first_name, middle_name, last_name,
            email, must_change_password, disabled, days_to_expiry, mins_to_unlock, default_warehouse,
            default_namespace, default_role, default_secondary_roles, mins_to_bypass_mfa, rsa_public_key,
            rsa_public_key_fp, rsa_public_key_2, rsa_public_key_2_fp, comment, type,
            enable_unredacted_query_syntax_error, network_policy are optional

        Examples
        ________
        Creating a user or altering one if it already exists:

        >>> user_parameters = User(
        ...     name="User1",
        ...     first_name="Snowy",
        ...     last_name="User",
        ...     must_change_password=False
        ...)
        >>> user_reference.create_or_alter(user_parameters)
        """
        self.collection._api.create_or_alter_user(user.name, user._to_model(), async_req=False)

    @api_telemetry
    def create_or_alter_async(self, user: User) -> PollingOperation[None]:
        """An asynchronous version of :func:`create_or_alter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.create_or_alter_user(user.name, user._to_model(), async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    @deprecated("drop")
    def delete(self) -> None:
        self.drop()

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop this user.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this user before dropping it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Deleting a user using its reference:

        >>> user_ref.drop()

        Deleting a user using its reference if it exists:

        >>> user_ref.drop(if_exists=True)
        """
        self.collection._api.delete_user(self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_user(self.name, if_exists=if_exists, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def fetch(self) -> User:
        """
        Fetch the details of a user.

        Examples
        ________
        Fetching a reference to a user to print its informations:

        >>> user_ref = root.users["test_user"].fetch()
        >>> print(user_ref.name, user_ref.first_name)
        """
        return User._from_model(self.collection._api.fetch_user(name=self.name, async_req=False))

    @api_telemetry
    def fetch_async(self) -> PollingOperation[User]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_user(name=self.name, async_req=True)
        return PollingOperation(future, lambda rest_model: User._from_model(rest_model))

    @api_telemetry
    def grant_role(self, role_type: str, role: Securable) -> None:
        """
        Grant a role to this user.

        Parameters
        __________
        role_type: str
            The type of role which would be granted.
        role: Securable
            The role which would be granted.

        Examples
        ________
        Using a user reference to grant a role to it:

        >>> user_reference.grant_role("role", Securable(name="test_role"))
        """
        grant = Grant(securable_type=role_type, securable=role)
        self.collection._api.grant(self.name, grant, async_req=False)

    @api_telemetry
    def grant_role_async(self, role_type: str, role: Securable) -> PollingOperation[None]:
        """An asynchronous version of :func:`grant_role`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(securable_type=role_type, securable=role)
        future = self.collection._api.grant(self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_role(self, role_type: str, role: Securable) -> None:
        """
        Revoke a role from this user.

        Parameters
        __________
        role_type: str
            The type of role which would be revoked.
        role: Securable
            The role which would be revoked.

        Examples
        ________
        Using a user reference to revoke a role from it:

        >>> user_reference.revoke_role("role", Securable(name="test_role"))
        """
        grant = Grant(securable_type=role_type, securable=role)
        self.collection._api.revoke_grants(self.name, grant, async_req=False)

    @api_telemetry
    def revoke_role_async(self, role_type: str, role: Securable) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_role`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(securable_type=role_type, securable=role)
        future = self.collection._api.revoke_grants(self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def iter_grants_to(self) -> Iterator[Grant]:
        """
        List grants to this user.

        Lists all roles granted to the user.

        Examples
        ________
        Using a user reference to list grants to it:

        >>> user_reference.iter_grants_to()
        """
        grants = self.collection._api.list_grants(self.name, async_req=False)
        return iter(grants)

    @api_telemetry
    def iter_grants_to_async(self) -> PollingOperation[Iterator[Grant]]:
        """An asynchronous version of :func:`iter_grants_to`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.list_grants(self.name, async_req=True)
        return PollingOperations.iterator(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on a user.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from a user.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for a user.

        Returns all tags assigned to a user. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(name=self.name, with_lineage=with_lineage, async_req=True)
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.api_integration._generated.api.api_integration_api import ApiIntegrationApi

__all__ = [
    "ApiIntegrationApi",
]



import logging
import typing

from typing import TYPE_CHECKING, Optional, Union

from pydantic import StrictBytes, StrictStr

from snowflake.connector import SnowflakeConnection
from snowflake.core import PollingOperation, RESTRoot
from snowflake.core.exceptions import UnexpectedResponseError

from .._generated.api_client import StoredProcApiClient
from .._internal.root_configuration import RootConfiguration
from .._rest_connection import RESTConnection
from ._generated.api import SparkConnectApi


if TYPE_CHECKING:
    from snowflake.snowpark import Session

logger = logging.getLogger(__name__)


class SparkConnectResource:
    """Represents a Spark Connect resource on GS.

    Note that there is no corresponding SparkConnectCollection class unlike other Snowflake objects e.g. Schema,
    Table etc. Spark Connect Resource does not represent a Snowflake object with metadata, so it doesn't need a
    "Collection" class to fetch or desc it. It is just a REST endpoint used by the Spark Connect gRPC service.

    Unlike other Snowflake Collection and Resource classes, SparkConnectResource is NOT instantiated by passing in a
    "Root" instance, instead it is instantiated by passing in an instance of a Session, SnowflakeConnection or
    RESTConnection. This makes it easy for the Spark Connect gRPC service to use it as a client as Spark Connect gRPC
    service is a multi-tenant service that receives session token with each gRPC call.

    Each method in the SparkConnectResource class maps one to one with a corresponding gRPC method in the Spark Connect
    gRPC service.
    """

    def __init__(
        self,
        connection: Union[RESTConnection, SnowflakeConnection, "Session"],
        root_config: Optional[RootConfiguration] = None,
    ) -> None:
        self._root = RESTRoot(connection, root_config)
        self._api = SparkConnectApi(
            root=self._root, resource_class=SparkConnectResource, sproc_client=StoredProcApiClient(root=self._root)
        )

    @property
    def root(self) -> "RESTRoot":
        return self._root

    def _assert_protobuf_response(self, response: typing.Any) -> bytearray:
        if not isinstance(response, bytearray):
            raise UnexpectedResponseError(self.root, response)
        return response

    def execute_plan(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.execute_plan(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def execute_plan_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`execute_plan`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.execute_plan(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def analyze_plan(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.analyze_plan(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def analyze_plan_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`analyze_plan`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.analyze_plan(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def config(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.config(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def config_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`config`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.config(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def add_artifacts(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.add_artifacts(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def add_artifacts_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`add_artifacts`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.add_artifacts(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def artifact_status(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.artifact_status(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def artifact_status_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`artifact_status`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.artifact_status(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def interrupt(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.interrupt(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def interrupt_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`interrupt`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.interrupt(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def reattach_execute(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.reattach_execute(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def reattach_execute_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`reattach_execute`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.reattach_execute(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))

    def release_execute(self, request: Union[StrictBytes, StrictStr]) -> bytearray:
        response = self._api.release_execute(body=request, async_req=False)
        return self._assert_protobuf_response(response)

    def release_execute_async(self, request: Union[StrictBytes, StrictStr]) -> PollingOperation[bytearray]:
        """An asynchronous version of :func:`release_execute`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.release_execute(body=request, async_req=True)
        return PollingOperation(future, lambda response: self._assert_protobuf_response(response))



import uuid

from typing import NamedTuple, TypedDict


UUID = str(uuid.uuid4()).replace("-", "_").upper()
TEST_DATABASE = f"INTEGRATION_TEST_{UUID}"
TEST_SCHEMA = f"GH_JOB_{UUID}"
TEST_WAREHOUSE = "TESTWH_PYTHON"
TEST_COMPUTE_POOL = "test_compute_pool"
TEST_IMAGE_REPO_DATABASE = "TESTDB_PYTHON_AUTO"
TEST_IMAGE_REPO_SCHEMA = "TESTSCHEMA_AUTO"
TEST_IMAGE_REPO_QUALIFIED_SCHEMA = f"{TEST_IMAGE_REPO_DATABASE}.{TEST_IMAGE_REPO_SCHEMA}"
TEST_IMAGE_REPO = "test_image_repo_auto"
DEFAULT_IR_URL = (
    "sfengineering-ss-lprpr-test2.registry.snowflakecomputing.com/"
    + f"{TEST_IMAGE_REPO_DATABASE.lower()}/{TEST_IMAGE_REPO_SCHEMA.lower()}/{TEST_IMAGE_REPO.lower()}"
)
TEST_SHARED_TAG_DATABASE = "TEST_SHARED_TAG_DATABASE"
TEST_SHARED_TAG_SCHEMA = "TEST_SHARED_TAG_SCHEMA"
TEST_SHARED_TAG_NAME = "TEST_SHARED_TAG_NAME"
TEST_SHARED_TAG_VALUE = "TEST_SHARED_TAG_VALUE"


class Tuple_database(NamedTuple):
    name: str
    param: str


class DatabaseDict(TypedDict):
    params: str
    schemas: set[str]


class SpcsSetupTuple(NamedTuple):
    instance_family: str
    compute_pool: str


objects_to_setup: dict[str, DatabaseDict] = {
    TEST_DATABASE: {"schemas": {TEST_SCHEMA}, "params": "DATA_RETENTION_TIME_IN_DAYS=1"}
}



"""
Snowflake Streamlit API.

The Snowflake Streamlit API is a REST API that you can use to access, update, and perform certain actions on Streamlit resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.streamlit._generated.models.streamlit_push_options import (
    StreamlitPushOptions,
)


class GitPushWithUsernamePassword(StreamlitPushOptions):
    """A model object representing the GitPushWithUsernamePassword resource.

    Constructs an object of type GitPushWithUsernamePassword with the provided properties.

    Parameters
    __________
    git_username : str
        A Git username.
    git_password : str
        A Git password.
    git_author_name : str
        The name of the Git author.
    git_author_email : str
        The email of the Git author.
    to_git_branch_uri : str, optional
        Pushes committed changes to the specified branch.
    git_push_comment : str, optional
        A comment to include in the Git push.
    """

    git_username: StrictStr

    git_password: StrictStr

    git_author_name: StrictStr

    git_author_email: StrictStr

    to_git_branch_uri: Optional[StrictStr] = None

    git_push_comment: Optional[StrictStr] = None

    __properties = ["auth_type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> GitPushWithUsernamePassword:
        """Create an instance of GitPushWithUsernamePassword from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["auth_type"] = StreamlitPushOptions.get_child_model_discriminator_value("GitPushWithUsernamePassword")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> GitPushWithUsernamePassword:
        """Create an instance of GitPushWithUsernamePassword from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return GitPushWithUsernamePassword.model_validate(obj)

        _obj = GitPushWithUsernamePassword.model_validate(
            {
                "git_username": obj.get("git_username"),
                "git_password": obj.get("git_password"),
                "git_author_name": obj.get("git_author_name"),
                "git_author_email": obj.get("git_author_email"),
                "to_git_branch_uri": obj.get("to_git_branch_uri"),
                "git_push_comment": obj.get("git_push_comment"),
            }
        )

        return _obj


class GitPushWithUsernamePasswordModel(StreamlitPushOptions):
    def __init__(
        self,
        git_username: str,
        git_password: str,
        git_author_name: str,
        git_author_email: str,
        # optional properties
        to_git_branch_uri: Optional[str] = None,
        git_push_comment: Optional[str] = None,
    ):
        """A model object representing the GitPushWithUsernamePassword resource.

        Constructs an object of type GitPushWithUsernamePassword with the provided properties.

        Parameters
        __________
        git_username : str
            A Git username.
        git_password : str
            A Git password.
        git_author_name : str
            The name of the Git author.
        git_author_email : str
            The email of the Git author.
        to_git_branch_uri : str, optional
            Pushes committed changes to the specified branch.
        git_push_comment : str, optional
            A comment to include in the Git push.
        """
        super().__init__()
        self.git_username = git_username
        self.git_password = git_password
        self.git_author_name = git_author_name
        self.git_author_email = git_author_email
        self.to_git_branch_uri = to_git_branch_uri
        self.git_push_comment = git_push_comment

    __properties = ["auth_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return GitPushWithUsernamePassword(
            git_username=self.git_username,
            git_password=self.git_password,
            git_author_name=self.git_author_name,
            git_author_email=self.git_author_email,
            to_git_branch_uri=self.to_git_branch_uri,
            git_push_comment=self.git_push_comment,
        )

    @classmethod
    def _from_model(cls, model) -> GitPushWithUsernamePasswordModel:
        return GitPushWithUsernamePasswordModel(
            git_username=model.git_username,
            git_password=model.git_password,
            git_author_name=model.git_author_name,
            git_author_email=model.git_author_email,
            to_git_branch_uri=model.to_git_branch_uri,
            git_push_comment=model.git_push_comment,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a GitPushWithUsernamePassword.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the GitPushWithUsernamePassword object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> GitPushWithUsernamePasswordModel:
        """Create an instance of GitPushWithUsernamePassword from a dict.

        This method constructs a GitPushWithUsernamePassword object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        GitPushWithUsernamePassword
            A GitPushWithUsernamePassword object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(GitPushWithUsernamePassword.from_dict(obj))


GitPushWithUsernamePassword._model_class = GitPushWithUsernamePasswordModel



"""Manages Snowflake preview test."""

from ._generated.models import (
    CreateResourceRequest,
    Resource,
    SharedModel,
)

__all__ = [
    "CreateResourceRequest",
    "Resource",
    "SharedModel",
]



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from snowflake.core.exceptions import UnauthorizedError
from tests.integ.account.utils import cleanup_accounts_completely, create_account

from ..utils import backup_role, random_string


pytestmark = [pytest.mark.internal_only, pytest.mark.skip_notebook]


def test_create_account(temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor):
    with backup_role(temp_customer_account_cursor):
        try:
            temp_customer_account_cursor.execute("use role orgadmin;")
            # test happy path
            account = create_account(temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor)

            # account already exists
            create_account(
                temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor, account, UnauthorizedError
            )

            # account deleted
            temp_customer_account_accounts[account.name].drop(grace_period_in_days=4)

            # account does not exist but is not available to create as it is still yet to be dropped
            create_account(
                temp_customer_account_accounts, sf_cursor, temp_customer_account_cursor, account, UnauthorizedError
            )

            # create account endpoint for rsa public key but since it is fake it will be rejected
            account.name = random_string(5, "test_create_account_")
            account.admin_rsa_public_key = "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAlmNyi9mEc18wdZw0lCeOxD3pwiLO9BzDIU84dqAws3hge4E2sgYaepcE3CUtC147jiWgD7atzKTHy0Ov88xh"  # pragma: allowlist secret
            create_account(
                temp_customer_account_accounts,
                sf_cursor,
                temp_customer_account_cursor,
                account,
                UnauthorizedError,
                ".*new public key rejected by current policy. reason: 'invalid public key'.*",
            )
        finally:
            # cleanup
            cleanup_accounts_completely(sf_cursor)



"""
Snowflake Tag API.

The Snowflake Tag API is a REST API that you can use to access, update, and perform certain actions on Tag resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Annotated


class Tag(BaseModel):
    """A model object representing the Tag resource.

    Constructs an object of type Tag with the provided properties.

    Parameters
    __________
    name : str
        Name of the tag
    allowed_values : list[str], optional
        Possible string values that can be assigned with the tag when the tag is set on an object
    propagate : str, optional
        Specifies that the tag will be automatically propagated from source objects to target objects
    on_conflict : str, optional
        Specifies what happens when there is a conflict between the values of propagated tags
    comment : str, optional
        Comment for the tag
    created_on : datetime, optional
        Date and time when the tag was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the tag is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the tag is stored — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the tag — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the tag — **Read-only:** *any user-provided value will be ignored.*
    """

    allowed_values: Optional[List[StrictStr]] = None

    propagate: Optional[StrictStr] = None

    on_conflict: Optional[StrictStr] = None

    comment: Optional[StrictStr] = None

    name: Annotated[str, Field(strict=True)]

    created_on: Optional[datetime] = None

    database_name: Optional[Annotated[str, Field(strict=True)]] = None

    schema_name: Optional[Annotated[str, Field(strict=True)]] = None

    owner: Optional[Annotated[str, Field(strict=True)]] = None

    owner_role_type: Optional[StrictStr] = None

    __properties = [
        "allowed_values",
        "propagate",
        "on_conflict",
        "comment",
        "name",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("database_name")
    def database_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("schema_name")
    def schema_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("owner")
    def owner_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Tag:
        """Create an instance of Tag from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "owner",
                    "owner_role_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Tag:
        """Create an instance of Tag from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Tag.model_validate(obj)

        _obj = Tag.model_validate(
            {
                "allowed_values": obj.get("allowed_values"),
                "propagate": obj.get("propagate"),
                "on_conflict": obj.get("on_conflict"),
                "comment": obj.get("comment"),
                "name": obj.get("name"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
            }
        )

        return _obj


class TagModel:
    def __init__(
        self,
        name: str,
        # optional properties
        allowed_values: Optional[list[str]] = None,
        propagate: Optional[str] = None,
        on_conflict: Optional[str] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
    ):
        """A model object representing the Tag resource.

        Constructs an object of type Tag with the provided properties.

        Parameters
        __________
        name : str
            Name of the tag
        allowed_values : list[str], optional
            Possible string values that can be assigned with the tag when the tag is set on an object
        propagate : str, optional
            Specifies that the tag will be automatically propagated from source objects to target objects
        on_conflict : str, optional
            Specifies what happens when there is a conflict between the values of propagated tags
        comment : str, optional
            Comment for the tag
        created_on : datetime, optional
            Date and time when the tag was created.
        database_name : str, optional
            Database in which the tag is stored
        schema_name : str, optional
            Schema in which the tag is stored
        owner : str, optional
            Role that owns the tag
        owner_role_type : str, optional
            The type of role that owns the tag
        """
        self.allowed_values = allowed_values
        self.propagate = propagate
        self.on_conflict = on_conflict
        self.comment = comment
        self.name = name
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner
        self.owner_role_type = owner_role_type

    __properties = [
        "allowed_values",
        "propagate",
        "on_conflict",
        "comment",
        "name",
        "created_on",
        "database_name",
        "schema_name",
        "owner",
        "owner_role_type",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Tag(
            allowed_values=self.allowed_values,
            propagate=self.propagate,
            on_conflict=self.on_conflict,
            comment=self.comment,
            name=self.name,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
        )

    @classmethod
    def _from_model(cls, model) -> TagModel:
        return TagModel(
            allowed_values=model.allowed_values,
            propagate=model.propagate,
            on_conflict=model.on_conflict,
            comment=model.comment,
            name=model.name,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Tag.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Tag object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TagModel:
        """Create an instance of Tag from a dict.

        This method constructs a Tag object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Tag
            A Tag object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Tag.from_dict(obj))


Tag._model_class = TagModel



import pytest

from snowflake.core.schema import SchemaResource


pytestmark = pytest.mark.usefixtures("backup_database_schema")


@pytest.mark.min_sf_ver("9.8.0")
def test_alter_all_params(schemas, temp_schema: SchemaResource):
    new_sc_def = temp_schema.fetch()
    new_sc_def.comment = "my new comment"
    new_sc_def.data_retention_time_in_days = 0
    new_sc_def.default_ddl_collation = "en_US-trim"
    #    new_sc_def.log_level = "INFO"
    #    new_sc_def.pipe_execution_paused = True
    new_sc_def.max_data_extension_time_in_days = 7
    new_sc_def.suspend_task_after_num_failures = 1
    #   new_sc_def.trace_level = "ALWAYS"
    new_sc_def.user_task_managed_initial_warehouse_size = "SMALL"
    new_sc_def.serverless_task_min_statement_size = "SMALL"
    new_sc_def.serverless_task_max_statement_size = "MEDIUM"
    new_sc_def.user_task_timeout_ms = 3600001
    temp_schema.create_or_alter(new_sc_def)
    new_sc = schemas[temp_schema.name].fetch()
    assert new_sc.name == new_sc_def.name
    assert new_sc.comment == new_sc_def.comment
    assert new_sc.data_retention_time_in_days == new_sc_def.data_retention_time_in_days
    assert new_sc.default_ddl_collation == new_sc_def.default_ddl_collation
    assert new_sc.options != "MANAGED ACCESS"
    # assert new_sc.log_level == new_sc_def.log_level
    # assert new_sc.pipe_execution_paused == new_sc_def.pipe_execution_paused
    assert new_sc.max_data_extension_time_in_days == new_sc_def.max_data_extension_time_in_days
    assert new_sc.suspend_task_after_num_failures == new_sc_def.suspend_task_after_num_failures
    #  assert new_sc.trace_level == new_sc_def.trace_level
    assert new_sc.user_task_managed_initial_warehouse_size == new_sc_def.user_task_managed_initial_warehouse_size
    assert new_sc.serverless_task_min_statement_size == new_sc_def.serverless_task_min_statement_size
    assert new_sc.serverless_task_max_statement_size == new_sc_def.serverless_task_max_statement_size
    assert new_sc.user_task_timeout_ms == new_sc_def.user_task_timeout_ms


@pytest.mark.use_accountadmin
def test_alter_with_managed_access(temp_schema: SchemaResource):
    new_sc_def = temp_schema.fetch()
    assert new_sc_def.managed_access is False

    # Test managed_access = True
    new_sc_def.managed_access = True
    temp_schema.create_or_update(new_sc_def)
    new_sc = temp_schema.fetch()
    assert new_sc.managed_access is True

    # Test managed_access = False
    new_sc_def.managed_access = None
    temp_schema.create_or_update(new_sc_def)
    new_sc = temp_schema.fetch()
    assert new_sc.managed_access is False



# coding: utf-8

"""
Preview Feature Test API

Test spec for preview feature filtering  # noqa: E501

The version of the OpenAPI document: 1.0.0
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Union, Optional


from datetime import datetime

from pydantic import BaseModel, ConfigDict, StrictStr

from typing import Any, ClassVar, Dict, List, Optional


class CreateResourceRequest(BaseModel):
    """A model object representing the CreateResourceRequest resource.

    Constructs an object of type CreateResourceRequest with the provided properties.

    Parameters
    __________
    name : str


    description : str, optional


    """

    name: StrictStr

    description: Optional[StrictStr] = None

    __properties = ["name", "description"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateResourceRequest:
        """Create an instance of CreateResourceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias"""

        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = dict(self._iter(to_dict=True, by_alias=True, exclude=exclude_properties, exclude_none=True))

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> CreateResourceRequest:
        """Create an instance of CreateResourceRequest from a dict."""

        if obj is None:
            return None

        if type(obj) is not dict:
            return CreateResourceRequest.parse_obj(obj)

        _obj = CreateResourceRequest.parse_obj(
            {
                "name": obj.get("name"),
                "description": obj.get("description"),
            }
        )

        return _obj


class CreateResourceRequestModel:
    def __init__(
        self,
        name: str,
        # optional properties
        description: Optional[str] = None,
    ):
        """A model object representing the CreateResourceRequest resource.

        Constructs an object of type CreateResourceRequest with the provided properties.

        Parameters
        __________
        name : str

        description : str, optional

        """

        self.name = name
        self.description = description

    __properties = ["name", "description"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return CreateResourceRequest(
            name=self.name,
            description=self.description,
        )

    @classmethod
    def _from_model(cls, model) -> CreateResourceRequestModel:
        return CreateResourceRequestModel(
            name=model.name,
            description=model.description,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a CreateResourceRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the CreateResourceRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> CreateResourceRequestModel:
        """Create an instance of CreateResourceRequest from a dict.

        This method constructs a CreateResourceRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        CreateResourceRequest
            A CreateResourceRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(CreateResourceRequest.from_dict(obj))


CreateResourceRequest._model_class = CreateResourceRequestModel



# mypy: ignore-errors

from concurrent.futures import Future
from datetime import time
from decimal import Decimal
from typing import TYPE_CHECKING, Any, Literal, Optional, Union, overload

from pydantic import StrictStr

from snowflake.core import PollingOperation

from .._internal.telemetry import api_telemetry
from .._internal.utils import deprecated
from ..exceptions import InvalidArgumentsError, InvalidResultError
from ._generated.api.function_api_base import FunctionCollectionBase, FunctionResourceBase
from ._generated.models.function_argument import FunctionArgument


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


def _cast_result(result: Any, returns: StrictStr) -> Any:
    if returns in ["NUMBER", "INT", "FIXED"]:
        return int(result)
    if returns == "REAL":
        return float(result)
    if returns == "DECFLOAT":
        return Decimal(result)
    if returns == "TEXT":
        return str(result)
    if returns == "TIME":
        return time(result)
    if returns == "BOOLEAN":
        return bool(int(result))
    return result


class FunctionCollection(FunctionCollectionBase):
    """Represents the collection operations on the Snowflake Function resource.

    With this collection, you can create, iterate through, and search for function that you have access to in the
    current context.

    Examples
    ________
    Creating a function instance:

    >>> functions = root.databases["my_db"].schemas["my_schema"].functions
    >>> new_function = Function(
    ...     name="foo",
    ...     returns="NUMBER",
    ...     arguments=[FunctionArgument(datatype="NUMBER")],
    ...     service="python",
    ...     endpoint="https://example.com",
    ...     path="example.py",
    ... )
    >>> functions.create(new_function)
    """

    _identifier_requires_args = True

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, FunctionResource)


class FunctionResource(FunctionResourceBase):
    """Represents a reference to a Snowflake function.

    With this function reference, you can fetch information about a function, as well as perform
    certain actions on it.
    """

    _identifier_requires_args = True
    _plural_name = "functions"

    def __init__(self, name_with_args: StrictStr, collection_class: FunctionCollection) -> None:
        super().__init__(name_with_args, collection_class)

    @api_telemetry
    @deprecated("drop")
    def delete(self, if_exists: bool = False) -> None:
        self.drop(if_exists=if_exists)

    @api_telemetry
    def execute(self, input_args: Optional[list[Any]] = None) -> Any:
        """Execute this function.

        Parameters
        __________
        input_args: list[Any], optional
            A list of arguments to pass to the function. The number of arguments must match the number of arguments
            the function expects.

        Examples
        ________
        Executing a function using its reference:

        >>> function_reference.execute(input_args=[1, 2, "word"])
        """
        return self._execute(input_args=input_args, async_req=False)

    @api_telemetry
    def execute_async(self, input_args: Optional[list[Any]] = None) -> PollingOperation[Any]:
        """An asynchronous version of :func:`execute`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return self._execute(input_args=input_args, async_req=True)

    @overload
    def _execute(self, input_args: Optional[list[Any]], async_req: Literal[True]) -> PollingOperation[Any]: ...

    @overload
    def _execute(self, input_args: Optional[list[Any]], async_req: Literal[False]) -> Any: ...

    def _execute(self, input_args: Optional[list[Any]], async_req: bool) -> Union[Any, PollingOperation[Any]]:
        function = self.fetch()
        args_count = len(function.arguments) if function.arguments is not None else 0

        if input_args is None:
            input_args = []

        if len(input_args) != args_count:
            raise InvalidArgumentsError(f"Function expects {args_count} arguments but received {len(input_args)}")

        function_args = []
        for i in range(args_count):
            argument = FunctionArgument()
            argument.value = input_args[i]
            argument.datatype = function.arguments[i].datatype
            function_args.append(argument)

        result_or_future = self.collection._api.execute_function(
            self.database.name, self.schema.name, function.name, function_args, async_req=async_req
        )

        def map_result(result: object) -> Any:
            if not isinstance(result, dict) or len(result.values()) != 1:
                raise InvalidResultError(f"Function result {str(result)} is invalid or empty")

            result = list(result.values())[0]
            return _cast_result(result, str(function.returns))

        if isinstance(result_or_future, Future):
            return PollingOperation(result_or_future, map_result)
        return map_result(result_or_future)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import datetime

import pytest

from pydantic_core._pydantic_core import ValidationError


pytestmark = [pytest.mark.skip_gov]


@pytest.mark.usefixtures("qa_mode_enabled")
def test_iter(compute_pools, temp_cp, instance_family):
    for cp in compute_pools.iter(like=temp_cp.name):
        assert cp.name == temp_cp.name.upper()
        assert cp.instance_family == instance_family.upper()
        assert cp.min_nodes == 1
        assert cp.max_nodes == 1
        assert not cp.auto_resume
        assert cp.comment == "created by temp_cp"
        assert cp.state in ("SUSPENDED", "STOPPING")
        assert cp.num_services == 0
        assert cp.num_jobs == 0
        assert cp.auto_suspend_secs == 600
        assert cp.active_nodes == 0
        assert cp.idle_nodes is None
        assert cp.target_nodes == 1
        assert cp.created_on == datetime.datetime(1967, 6, 23, 7, 0, 0, 123000, tzinfo=datetime.timezone.utc)
        assert cp.resumed_on == datetime.datetime(1967, 6, 23, 7, 0, 0, 123000, tzinfo=datetime.timezone.utc)
        assert cp.updated_on == datetime.datetime(1967, 6, 23, 7, 0, 0, 123000, tzinfo=datetime.timezone.utc)
        assert cp.owner == "ACCOUNTADMIN"
        assert cp.is_exclusive is False
        assert cp.application is None
        assert cp.budget is None

    compute_pools_names = [cp.name for cp in compute_pools.iter(like="test_%")]
    assert temp_cp.name.upper() in compute_pools_names

    compute_pools_names = [cp.name for cp in compute_pools.iter(starts_with="test_")]
    assert temp_cp.name.upper() not in compute_pools_names

    compute_pools_names = [cp.name for cp in compute_pools.iter(starts_with="TEST_")]
    assert temp_cp.name.upper() in compute_pools_names


def test_iter_limit(compute_pools):
    with pytest.raises(ValidationError):
        assert len(compute_pools.iter(starts_with="TEST_", limit=0)) == 0

    compute_pools_names = [cp.name for cp in compute_pools.iter(starts_with="TEST_", limit=1)]
    assert len(compute_pools_names) <= 1

    with pytest.raises(ValidationError):
        compute_pools.iter(starts_with="TEST_", limit=10001)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from tests.integ.managed_account.utils import cleanup_managed_accounts_completely, create_managed_account

from ..utils import backup_role


pytestmark = [pytest.mark.internal_only, pytest.mark.skip_notebook]


def test_list_managed_account(temp_customer_account_managed_accounts, sf_cursor, temp_customer_account_cursor):
    with backup_role(temp_customer_account_cursor):
        try:
            temp_customer_account_cursor.execute("use role accountadmin;")
            # test happy path
            managed_account = create_managed_account(
                temp_customer_account_managed_accounts, sf_cursor, temp_customer_account_cursor
            )

            temp_customer_account_managed_accounts[managed_account.name].drop()

            managed_account = create_managed_account(
                temp_customer_account_managed_accounts, sf_cursor, temp_customer_account_cursor
            )
            managed_account_2 = create_managed_account(
                temp_customer_account_managed_accounts, sf_cursor, temp_customer_account_cursor
            )

            managed_account_names = [
                managed_account.name for managed_account in temp_customer_account_managed_accounts.iter()
            ]
            assert len(managed_account_names) == 2

            assert ["This is my managed account@$W%*#$()%", "This is my managed account@$W%*#$()%"] == [
                managed_account.comment for managed_account in temp_customer_account_managed_accounts.iter()
            ]

            managed_account_names = [
                managed_account.name for managed_account in temp_customer_account_managed_accounts.iter(like="")
            ]
            assert len(managed_account_names) == 0

            managed_account_names = [
                managed_account.name
                for managed_account in temp_customer_account_managed_accounts.iter(like="TEST_CREATE_MANAGED_ACCOUNT_%")
            ]
            assert len(managed_account_names) == 2

            # cleanup
            temp_customer_account_managed_accounts[managed_account.name].drop()
            temp_customer_account_managed_accounts[managed_account_2.name].drop()
        finally:
            # cleanup
            cleanup_managed_accounts_completely(temp_customer_account_cursor)



def test_resume_and_suspend(dynamic_table_handle, dynamic_tables):
    dynamic_tables["dummy___table"].suspend(if_exists=True)
    dynamic_tables["dummy___table"].resume(if_exists=True)
    assert dynamic_table_handle.fetch().scheduling_state == "RUNNING"
    dynamic_table_handle.suspend()
    assert dynamic_table_handle.fetch().scheduling_state == "SUSPENDED"
    dynamic_table_handle.resume()
    assert dynamic_table_handle.fetch().scheduling_state == "RUNNING"


def test_refresh(dynamic_table_handle, dynamic_tables):
    dynamic_tables["dummy___table"].refresh(if_exists=True)
    dynamic_table_handle.refresh()



import pytest

from snowflake.core._internal.snowapi_parameters import SnowApiParameter, SnowApiParameters


VALUES = [
    ("true", True),
    ("TRUE", True),
    ("True", True),
    ("t", True),
    ("yes", True),
    ("y", True),
    ("on", True),
    ("any-other-value", False),
]


def test_max_threads():
    parameters = SnowApiParameters({SnowApiParameter.MAX_THREADS: "30"})
    assert parameters.max_threads == 30


def test_default_max_threads():
    parameters = SnowApiParameters({SnowApiParameter.MAX_THREADS: None})
    assert parameters.max_threads is None


@pytest.mark.parametrize("flag", ("should_retry_request", "should_print_verbose_stack_trace", "fix_hostname"))
@pytest.mark.parametrize("value, expected", VALUES)
def test_flags(flag, value, expected):
    parameters = SnowApiParameters(
        {
            SnowApiParameter.USE_CLIENT_RETRY: value,
            SnowApiParameter.PRINT_VERBOSE_STACK_TRACE: value,
            SnowApiParameter.FIX_HOSTNAME: value,
        }
    )
    assert getattr(parameters, flag) is expected


def test_default_flags():
    parameters = SnowApiParameters({})
    assert parameters.should_retry_request is True
    assert parameters.should_print_verbose_stack_trace is True
    assert parameters.fix_hostname is True

    parameters = SnowApiParameters(
        {
            SnowApiParameter.USE_CLIENT_RETRY: None,
            SnowApiParameter.PRINT_VERBOSE_STACK_TRACE: None,
            SnowApiParameter.FIX_HOSTNAME: None,
        }
    )
    assert parameters.should_retry_request is True
    assert parameters.should_print_verbose_stack_trace is True
    assert parameters.fix_hostname is True
